from social_validator import shared
from social_validator.exceptions import ValidationError

# Restrictions for username
USERNAME_MIN_LENGTH = 3
USERNAME_MAX_LENGTH = 30

# YouTube has two types of links leading to a channel.
# The first type is displayed with an "@" symbol and contains the exact address
# with hyphens, underscores, and dots.
# The second type doesn't contain the "@" symbol and ignores any hyphens and
# underscores. The dots in this type also have significance.
_IGNORED_CHARACTERS = frozenset(("_", "-"))

RESERVED_USERNAMES = frozenset(
    (
        "blog",
        "browse",
        "channels",
        "explore",
        "favorites",
        "feed",
        "gaming",
        "help",
        "kids",
        "live",
        "logout",
        "movies",
        "music",
        "playlist",
        "search",
        "settings",
        "signin",
        "signup",
        "sports",
        "support",
        "trending",
        "upload",
        "video",
        "videos",
        "watch",
    )
)

# Restriction for videos
VIDEO_ID_LENGTH = 11
VIDEO_NAME_MIN_LENGTH = 1
VIDEO_NAME_MAX_LENGTH = 100
VIDEO_DESCRIPTION_SHORT_MAX_LENGTH = 135
VIDEO_DESCRIPTION_MAX_LENGTH = 5000
VIDEO_COMMENT_MIN_LENGTH = 1
VIDEO_COMMENT_MAX_LENGTH = 10000


def _replace_ignored_chars(s: str, replace: str) -> str:
    """
    Replaces all characters from the :py:const:`_IGNORED_CHARACTERS` with the
    specified one.
    """
    for char in _IGNORED_CHARACTERS:
        s = s.replace(char, replace)

    return s


def _contains_angle_brackets(s: str) -> bool:
    """
    Checks for the presence of angle brackets within a string.
    """
    return '>' in s or '<' in s


def _is_valid_text(text: str) -> bool:
    """
    Checks the text for compliance with YouTube restrictions.
    """
    return not _contains_angle_brackets(text) and text.isprintable()


def is_valid_username(username: str, *, strct: bool = True) -> bool:
    """
    Checks the given username for length restrictions and character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_username`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    username = _replace_ignored_chars(username, "_" if strct else "")

    return (
        USERNAME_MIN_LENGTH <= len(username) <= USERNAME_MAX_LENGTH
        and username[0] != "."
        and not username.isdigit()
        and shared.is_valid_id(username.replace(".", "_"))
    )


def is_reserved_username(username: str) -> bool:
    """
    Checks whether the username is reserved.
    Case-insensitive.

    Refer to the :py:const:`RESERVED_USERNAMES` for the complete list of words.

    :return: ``True`` if **username** is reserved, ``False`` otherwise
    """
    return username.lower() in RESERVED_USERNAMES


def is_valid_video_id(_id: str) -> bool:
    """
    Checks the given ID for length restrictions and character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_video_id`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return (
        len(_id) == VIDEO_ID_LENGTH
        and shared.is_valid_id(_id.replace('-', '_'))
    )


def is_valid_video_name(text: str) -> bool:
    """
    Checks the given text for length restrictions and character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_video_name`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return (
        VIDEO_NAME_MIN_LENGTH <= len(text) <= VIDEO_NAME_MAX_LENGTH
        and _is_valid_text(text)
    )


def is_valid_video_comment(text: str) -> bool:
    """
    Checks the given text for length restrictions and character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_video_comment`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return (
        VIDEO_COMMENT_MIN_LENGTH <= len(text) <= VIDEO_COMMENT_MAX_LENGTH
        and text.isprintable()
    )


def is_valid_video_description(text: str, *, short: bool = False) -> bool:
    """
    Checks the given text for character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_video_description`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return (
        len(text) <= (VIDEO_DESCRIPTION_MAX_LENGTH if not short else VIDEO_DESCRIPTION_SHORT_MAX_LENGTH)
        and _is_valid_text(text)
    )


def validate_username(username: str, *, strict: bool = True) -> str:
    """
    Validates a username based on the following criteria:

    - It contains between 3 and 30 characters, inclusively;
    - It consist of A-Za-z, digits, hyphens, underscores, and dots;
    - It doesn't start with a dot;
    - It is not entirely composed of digits;
    - It is not a reserved word.

    All reserved words are listed in :py:const:`RESERVED_USERNAMES`

    It is important to highlight the "strict" argument. It determines whether
    the strict validation will be enabled or not.
    When it is enabled, the validation will be strict and all characters will
    be preserved.
    When it is disabled, a soft validation will be performed,
    ignoring dashes and underscores.

    This is due to the specific algorithms used by YouTube. When following a
    link that includes the "@" symbol, a strict search will be conducted.
    Otherwise, a soft search will be used.

    :param username: User text ID
    :param strict: Is the username includes "@" symbol
    :return: Input username, converted to lower-case. If the strict mode is
        disabled, all hyphens and underscores will be removed.
    :raise ValidationError: If the username doesn't meet the criteria
    """
    if not is_valid_username(username, strct=strict):
        raise ValidationError(
            f"Username must have between {USERNAME_MIN_LENGTH} and "
            f"{USERNAME_MAX_LENGTH} characters, including hyphens, "
            "underscores and dots, can't consist entirely of numbers, and can't "
            "start with a dot",
            input_value=username,
            strict=strict,
        )
    elif is_reserved_username(username):
        raise ValidationError(
            "Username must not be a reserved word",
            input_value=username,
            strict=strict,
        )

    if not strict:
        # Replace them because they are not considered by YouTube
        username = _replace_ignored_chars(username, "")

    return username.lower()


def validate_video_id(_id: str) -> str:
    """
    Validates a URL based on the following criteria:

    - It has a length of 11 characters;
    - It only contains the characters A-Za-z, 0-9, dashes, and underscores.

    :param _id: Value that stored in the argument "v" (/watch?v=)
    :return: Input value
    :raise ValidationError: If the URL doesn't meet the criteria
    """
    if not is_valid_video_id(_id):
        raise ValidationError(
            f'The ID must be exactly {VIDEO_ID_LENGTH} characters long and '
            f'can only contain alphanumeric characters (A-Z, a-z, 0-9), '
            f'dashes, and underscores',
            input_value=_id,
        )

    return _id


def validate_video_name(text: str) -> str:
    """
    Validates a text based on the following criteria:

    - It has a length of 1 to 100 characters, inclusive;
    - It consists of displayable characters;
    - It not contains angle brackets (>, <) within it.

    :param text: Video name
    :return: Input value
    :raise ValidationError: If the text doesn't meet the criteria
    """
    if not is_valid_video_name(text):
        raise ValidationError(
            f'The video name must have a length of {VIDEO_NAME_MIN_LENGTH} to '
            f'{VIDEO_NAME_MAX_LENGTH} characters, consist of displayable '
            f'characters and not have angle brackets within it',
            input_value=text,
        )

    return text


def validate_video_description(text: str, *, short: bool = False) -> str:
    """
    Validates a text based on the following criteria:

    - It has a length up to 5000 (or 135 for short) characters long;
    - it consists of displayable characters;
    - It not contains angle brackets (>, <) within it.

    :param text: Description text
    :param short: Is it short version
    :return: Input value
    :raise ValidationError: If the text doesn't meet the criteria
    """
    if not is_valid_video_description(text, short=short):
        raise ValidationError(
            f'The video description must be up to '
            f'{VIDEO_DESCRIPTION_MAX_LENGTH if not short else VIDEO_DESCRIPTION_SHORT_MAX_LENGTH} '
            f'characters long, not contain angle brackets, '
            f'and not have escaped characters or other special characters',
            input_value=text,
            short=short,
        )

    return text


def validate_video_comment(text: str) -> str:
    """
    Validates a text based on the following criteria:

    - It has a length of 1 to 10000 characters, inclusive;
    - it consists of displayable characters;

    :param text: Comment text
    :return: Input value
    :raise ValidationError: If the text doesn't meet the criteria
    """
    if not is_valid_video_comment(text):
        raise ValidationError(
            f'The video comment must have a length of '
            f'{VIDEO_COMMENT_MIN_LENGTH} to {VIDEO_COMMENT_MAX_LENGTH} '
            f'characters and consist of displayable characters',
            input_value=text,
        )

    return text



from social_validator import twitch
from tests.shared.input import ESCAPED_STRING, RANDOM_UNICODE_STRING

VALID_USERNAMES = {
    "a" * twitch.USERNAME_MIN_LENGTH,
    "a" * twitch.USERNAME_MAX_LENGTH,
    "a123",
    "123a",
    "a_12",
    "username",
    "user_name",
    "user___name",
    "u_s_e_r_n_a_m_e",
    "u_1_2_3_4_5_6",
    "username_______",
}

INVALID_USERNAMES = {
    "",  # empty
    "a" * (twitch.USERNAME_MIN_LENGTH - 1),  # too short
    "a" * (twitch.USERNAME_MAX_LENGTH + 1),  # too long
    "_username",  # starts with underscore
    "_" * twitch.USERNAME_MIN_LENGTH,  # consist entirely of special chars
    "123456",  # consist entirely of digits
    "user name",  # special char
    "username@",  # special char
    RANDOM_UNICODE_STRING,  # invalid chars
    ESCAPED_STRING,  # escaped chars
}

VALID_DESCRIPTIONS = {
    "",
    "a",
    "1" * twitch.DESCRIPTION_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
}

INVALID_DESCRIPTIONS = {
    "1" * (twitch.DESCRIPTION_MAX_LENGTH + 1),  # too long
    ESCAPED_STRING,  # escaped chars
}

VALID_MESSAGES = {
    "a",
    "1" * twitch.MESSAGE_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
}

INVALID_MESSAGES = {
    "",  # empty
    "1" * (twitch.MESSAGE_MAX_LENGTH + 1),  # too long
    ESCAPED_STRING,  # escaped chars
}

# Combine lower and upper cases
RESERVED_USERNAMES = list(twitch.RESERVED_USERNAMES) + [
    word.upper() for word in twitch.RESERVED_USERNAMES
]



def is_valid_id(s: str) -> bool:
    """
    Checks the string for matching the pattern: A-Za-z, 0-9, _
    """
    # use a small hack to avoid performing additional checks
    s = s.replace("_", "A")
    return s.isascii() and s.isalnum()



import pytest
from assertpy import assert_that

from social_validator import twitch
from social_validator.exceptions import ValidationError
from tests.twitch import input


@pytest.mark.parametrize("username", input.VALID_USERNAMES)
def test_valid_username(username: str) -> None:
    assert_that(twitch.validate_username(username)).is_equal_to(username)


@pytest.mark.parametrize("username", input.INVALID_USERNAMES)
def test_invalid_username(username: str) -> None:
    with pytest.raises(ValidationError):
        twitch.validate_username(username)


@pytest.mark.parametrize("username", input.RESERVED_USERNAMES)
def test_reserved_username(username: str) -> None:
    with pytest.raises(ValidationError):
        twitch.validate_username(username)


@pytest.mark.parametrize("text", input.VALID_DESCRIPTIONS)
def test_valid_description(text: str) -> None:
    assert_that(twitch.validate_description(text)).is_equal_to(text)


@pytest.mark.parametrize("text", input.INVALID_DESCRIPTIONS)
def test_invalid_description(text: str) -> None:
    with pytest.raises(ValidationError):
        twitch.validate_description(text)


@pytest.mark.parametrize("text", input.VALID_MESSAGES)
def test_valid_message(text: str) -> None:
    assert_that(twitch.validate_message(text)).is_equal_to(text)


@pytest.mark.parametrize("text", input.INVALID_MESSAGES)
def test_invalid_message(text: str) -> None:
    with pytest.raises(ValidationError):
        twitch.validate_message(text)



from social_validator import shared
from social_validator.exceptions import ValidationError

# Restrictions for usernames
USERNAME_MIN_LENGTH = 4
USERNAME_MAX_LENGTH = 25

RESERVED_USERNAMES = frozenset(
    {
        "about",
        "bits",
        "blog",
        "broadcast",
        "chat",
        "clips",
        "directory",
        "downloads",
        "events",
        "following",
        "game",
        "giftcard",
        "help",
        "host",
        "jobs",
        "legal",
        "live",
        "login",
        "moderator",
        "music",
        "partner",
        "point",
        "prime",
        "privacy",
        "raid",
        "redeem",
        "search",
        "security",
        "settings",
        "signup",
        "store",
        "stream",
        "subscriptions",
        "support",
        "team",
        "twitch",
        "twitchcon",
        "turbo",
        "user",
        "videos",
    }
)

# Restrictions for description (about) field
DESCRIPTION_MAX_LENGTH = 300

# Restrictions for messages
MESSAGE_MIN_LENGTH = 1
MESSAGE_MAX_LENGTH = 500


def is_valid_username(username: str) -> bool:
    """
    Checks the given username for character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_username`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return (
        USERNAME_MIN_LENGTH <= len(username) <= USERNAME_MAX_LENGTH
        and username[0] != "_"
        and not username.isdigit()
        and shared.is_valid_id(username)
    )


def is_reserved_username(username: str) -> bool:
    """
    Checks whether the username is reserved.
    The complete list of reserved words can be viewed in the
    :py:const:`RESERVED_USERNAMES`

    :return: ``True``, if username is reserved, otherwise ``False``.
    """
    return username.lower() in RESERVED_USERNAMES


def is_valid_description(text: str) -> bool:
    """
    Checks the given text for character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_description`.

    :return: ``True``, if all the conditions are met, otherwise ``False``.
    """
    return len(text) <= DESCRIPTION_MAX_LENGTH and text.isprintable()


def is_valid_message(text: str) -> bool:
    """
    Checks the given text for character correctness.
    More detailed validation criteria are described in
    :py:func:`validate_message`.

    :return: ``True``, if all the conditions are met, otherwise ``False``
    """
    return MESSAGE_MIN_LENGTH <= len(text) <= MESSAGE_MAX_LENGTH and text.isprintable()


def validate_username(username: str) -> str:
    """
    Validates a username based on the following criteria:

    - The length of the username must be 4 to 25 characters long;
    - The username must not start with an underscore;
    - The username must not consist entirely of digits;
    - The username must consist of: A-Za-z, 0-9 and underscores;
    - The username must not be a reserved word.

    All reserved words are listed in :py:const:`RESERVED_USERNAMES`

    :param username: User text ID
    :return: Input username
    :raise ValidationError: If the username does not meet the criteria
    """
    if not is_valid_username(username):
        raise ValidationError(
            "Username must be 4 to 25 characters long, exclude the underscore as the first "
            "character, and can only consist of: A-Za-z, 0-9 and underscores",
            input_value=username,
        )

    if is_reserved_username(username):
        raise ValidationError(
            "Username must not be a reserved word",
            input_value=username,
        )

    return username


def validate_description(text: str) -> str:
    """
    Validates a text based on the following criteria:

    - The text must not exceed 300 characters;
    - The text must not be escaped.

    :param text: Description text
    :return: Input text
    :raise ValidationError: if the text does not meet the criteria
    """
    if not is_valid_description(text):
        raise ValidationError(
            "Description must not exceed 300 characters and must not be escaped",
            input_value=text,
        )

    return text


def validate_message(text: str) -> str:
    """
    Validates a text based on the following criteria:

    - The text must be between 1 and 500 characters, inclusive;
    - The text must not be escaped.

    :param text: Message text
    :return: Input text
    :raise ValidationError: if the text does not meet the criteria
    """
    if not is_valid_message(text):
        raise ValidationError(
            "Message must contain between 1 and 500 characters, inclusive, "
            "and must not be escaped",
            input_value=text,
        )

    return text



from social_validator import telegram
from tests.shared.input import ESCAPED_STRING, RANDOM_UNICODE_STRING

VALID_IDS = (
    "A" * telegram.ID_MAX_LENGTH,
    "A" * telegram.ID_MIN_LENGTH,
    "B12345",
    "Z12345",
    "Q_t_e_s_t",
    "Q_1_2_3_4_5",
)

INVALID_IDS = (
    "",
    "A" * (telegram.ID_MIN_LENGTH - 1),
    "A" * (telegram.ID_MAX_LENGTH + 1),
    "123456",
    "_test",
    "__test",
    "test__",
    "t__q__",
    "t______",
    "1_test",
    "тест_ид",
    RANDOM_UNICODE_STRING,
    ESCAPED_STRING,
)

VALID_BOT_IDS = (
    f"{'A' * 2}bot",
    f"{'A' * (telegram.ID_MAX_LENGTH - 3)}bot",
    "B12345bot",
    "Z12345_bot",
    "Q_1_2_3_4_5bot",
)

INVALID_BOT_IDS = (
    "Abot",
    f"{'A' * (telegram.ID_MAX_LENGTH - 2)}bot",
    "123456bot",
    "_testbot",
    "__testbot",
    "test__bot",
    "t__q__bot",
    "t______bot",
    "1_testbot",
    "тестовый_идbot",
    "correct_name_without_suffix",
    f"{RANDOM_UNICODE_STRING}bot",
    ESCAPED_STRING,
)

VALID_DESCRIPTIONS = (
    ("", "user"),
    ("a" * telegram.DESCRIPTION_USER_MAX_LENGTH, "user"),
    (RANDOM_UNICODE_STRING, "user"),
    ("", "group"),
    ("a" * telegram.DESCRIPTION_GROUP_MAX_LENGTH, "group"),
    (RANDOM_UNICODE_STRING, "group"),
    ("", "channel"),
    ("a" * telegram.DESCRIPTION_CHANNEL_MAX_LENGTH, "channel"),
    (RANDOM_UNICODE_STRING, "channel"),
    ("", "bot"),
    ("a" * telegram.DESCRIPTION_BOT_MAX_LENGTH, "bot"),
    (RANDOM_UNICODE_STRING, "bot"),
)

INVALID_DESCRIPTIONS = (
    ("A" * (telegram.DESCRIPTION_USER_MAX_LENGTH + 1), "user"),
    ("A" * (telegram.DESCRIPTION_GROUP_MAX_LENGTH + 1), "group"),
    ("A" * (telegram.DESCRIPTION_CHANNEL_MAX_LENGTH + 1), "channel"),
    ("A" * (telegram.DESCRIPTION_BOT_MAX_LENGTH + 1), "bot"),
    (ESCAPED_STRING, "user"),
    (ESCAPED_STRING, "group"),
    (ESCAPED_STRING, "channel"),
    (ESCAPED_STRING, "bot"),
)

INVALID_DESCRIPTIONS_CHAT_TYPES = (
    ("test", ""),
    ("test", "unexpected"),
)

VALID_CHAT_NAMES = (
    "A",
    "A" * telegram.CHAT_NAME_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)

INVALID_CHAT_NAMES = (
    "",
    "A" * (telegram.CHAT_NAME_MAX_LENGTH + 1),
    ESCAPED_STRING,
)

VALID_FIRST_NAMES = (
    "A",
    "A" * telegram.FIRST_NAME_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)

INVALID_FIRST_NAMES = (
    "",
    "A" * (telegram.FIRST_NAME_MAX_LENGTH + 1),
    ESCAPED_STRING,
)

VALID_LAST_NAMES = (
    "",
    "A" * telegram.LAST_NAME_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)

INVALID_LAST_NAMES = ("A" * (telegram.LAST_NAME_MAX_LENGTH + 1), ESCAPED_STRING)

VALID_FULL_NAMES = (
    # this may be a bit hard to read,
    # but here we just map each first_name to each last_name
    *(
        (first_name, last_name)
        for last_name in VALID_LAST_NAMES
        for first_name in VALID_FIRST_NAMES
    ),
    # map valid first names to empty last name
    *((first_name, "") for first_name in VALID_FIRST_NAMES),
)

INVALID_FULL_NAMES = (
    # this may be even more difficult to read,
    # but here we simply map the correct and incorrect first and last names
    # to get a validation error in the final result
    *(
        (first_name, last_name)
        for last_name in INVALID_LAST_NAMES
        for first_name in VALID_FIRST_NAMES
    ),
    *(
        (first_name, last_name)
        for last_name in VALID_LAST_NAMES
        for first_name in INVALID_FIRST_NAMES
    ),
    *(
        (first_name, last_name)
        for last_name in INVALID_LAST_NAMES
        for first_name in INVALID_FIRST_NAMES
    ),
    # map invalid first names to empty last name
    *((first_name, "") for first_name in INVALID_FIRST_NAMES),
)

VALID_MESSAGES = (
    # (text: str, include_media: bool)
    ("A", False),
    ("A" * telegram.MESSAGE_MAX_LENGTH, False),
    (RANDOM_UNICODE_STRING, False),
    ("A", True),
    ("A" * telegram.MEDIA_MESSAGE_MAX_LENGTH, True),
    (RANDOM_UNICODE_STRING, True),
)

INVALID_MESSAGES = (
    # (text: str, include_media: bool)
    ("", False),
    ("A" * (telegram.MESSAGE_MAX_LENGTH + 1), False),
    ("", True),
    ("A" * (telegram.MEDIA_MESSAGE_MAX_LENGTH + 1), True),
    (ESCAPED_STRING, True),
    (ESCAPED_STRING, False),
)

VALID_COMMANDS = (
    "A",
    "A" * telegram.COMMAND_MAX_LENGTH,
    "12345",
    "_",
    "_____",
    "_test",
    "__test",
    "_12345",
    "__12345",
    "another__test__",
    "double_underscore__",
    "t_e_s_t_u_n_d_e_r_s_c_o_r_e_s",
    "1_2_3_4_5",
    "test12345withDIGITS",
)

INVALID_COMMANDS = (
    "",
    "A" * (telegram.COMMAND_MAX_LENGTH + 1),
    RANDOM_UNICODE_STRING,
    ESCAPED_STRING,
)

# combine a lower and upper case
RESERVED_IDS = list(telegram.RESERVED_IDS) + [w.upper() for w in telegram.RESERVED_IDS]
RESERVED_START_IDS = list(telegram.RESERVED_START_IDS) + [
    w.upper() for w in telegram.RESERVED_START_IDS
]



import pytest as pytest
from assertpy import assert_that

from social_validator import youtube
from social_validator.exceptions import ValidationError
from tests.youtube import input


@pytest.mark.parametrize('username', input.VALID_STRICT_USERNAMES)
def test_valid_strict_username(username: str) -> None:
    assert_that(youtube.validate_username(username, strict=True)).is_equal_to(username.lower())


@pytest.mark.parametrize('username', input.VALID_SOFT_USERNAMES)
def test_valid_soft_username(username: str) -> None:
    expected = username.replace('-', '').replace('_', '').lower()
    assert_that(youtube.validate_username(username, strict=False)).is_equal_to(expected)


@pytest.mark.parametrize('username', input.INVALID_STRICT_USERNAMES)
def test_invalid_strict_username(username: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_username(username, strict=True)
    assert_that(e.value.input_value).is_equal_to(username)
    assert_that(e.value.arguments).contains('strict')


@pytest.mark.parametrize('username', input.INVALID_SOFT_USERNAMES)
def test_invalid_soft_username(username: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_username(username, strict=False)
    assert_that(e.value.input_value).is_equal_to(username)
    assert_that(e.value.arguments).contains('strict')


@pytest.mark.parametrize('username', input.RESERVED_USERNAMES)
def test_reserved_username(username: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_username(username)
    assert_that(e.value.input_value).is_equal_to(username)


@pytest.mark.parametrize('_id', input.VALID_VIDEO_IDS)
def test_valid_video_id(_id: str) -> None:
    assert_that(youtube.validate_video_id(_id)).is_equal_to(_id)


@pytest.mark.parametrize('_id', input.INVALID_VIDEO_IDS)
def test_invalid_video_id(_id: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_video_id(_id)
    assert_that(e.value.input_value).is_equal_to(_id)


@pytest.mark.parametrize('text', input.VALID_VIDEO_NAMES)
def test_valid_video_name(text: str) -> None:
    assert_that(youtube.validate_video_name(text)).is_equal_to(text)


@pytest.mark.parametrize('text', input.INVALID_VIDEO_NAMES)
def test_invalid_video_name(text: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_video_name(text)
    assert_that(e.value.input_value).is_equal_to(text)


@pytest.mark.parametrize('text', input.VALID_VIDEO_DESCRIPTIONS)
def test_valid_video_description(text: str) -> None:
    assert_that(youtube.validate_video_description(text, short=False)).is_equal_to(text)


@pytest.mark.parametrize('text', input.VALID_VIDEO_DESCRIPTIONS_SHORT)
def test_valid_video_description_short(text: str) -> None:
    assert_that(youtube.validate_video_description(text, short=True)).is_equal_to(text)


@pytest.mark.parametrize('text', input.INVALID_VIDEO_DESCRIPTIONS)
def test_invalid_video_description(text: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_video_description(text, short=False)
    assert_that(e.value.input_value).is_equal_to(text)
    assert_that(e.value.arguments).contains('short')


@pytest.mark.parametrize('text', input.INVALID_VIDEO_DESCRIPTIONS_SHORT)
def test_invalid_video_description_short(text: str) -> None:
    with pytest.raises(ValidationError) as e:
        youtube.validate_video_description(text, short=True)
    assert_that(e.value.input_value).is_equal_to(text)
    assert_that(e.value.arguments).contains('short')


@pytest.mark.parametrize('text', input.VALID_VIDEO_COMMENTS)
def test_valid_video_comment(text: str) -> None:
    assert_that(youtube.validate_video_comment(text)).is_equal_to(text)


@pytest.mark.parametrize('text', input.INVALID_VIDEO_COMMENTS)
def test_invalid_video_comment(text: str) -> None:
    with pytest.raises(ValidationError) as e:
        assert_that(youtube.validate_video_comment(text))
    assert_that(e.value.input_value).is_equal_to(text)



from typing import Literal, Tuple

from social_validator import shared
from social_validator.exceptions import ValidationError

# Restrictions for user, channel and bot identifiers
ID_MIN_LENGTH = 5
ID_MAX_LENGTH = 32

RESERVED_IDS = frozenset(
    [
        "anonymous",
        "awesome",
        "brilliant",
        "career",
        "community",
        "contact",
        "download",
        "events",
        "feedback",
        "helper",
        "media",
        "might",
        "moderator",
        "notifications",
        "official",
        "partners",
        "press",
        "private",
        "public",
        "report",
        "share",
        "start",
        "staff",
        "terms",
        "update",
        "updates",
        "video",
        "warning",
        "watch",
    ]
)
# Reserved words, from which an identifier can't begin
RESERVED_START_IDS = frozenset(
    [
        "admin",
        "support",
        "telegram",
        "settings",
    ]
)

# Restrictions for description (about) field
DESCRIPTION_USER_MAX_LENGTH = 70
DESCRIPTION_GROUP_MAX_LENGTH = 255
DESCRIPTION_CHANNEL_MAX_LENGTH = DESCRIPTION_GROUP_MAX_LENGTH
DESCRIPTION_BOT_MAX_LENGTH = 120

# Restrictions for chat names
CHAT_NAME_MIN_LENGTH = 1
CHAT_NAME_MAX_LENGTH = 128

# Restrictions for user and bot names
FIRST_NAME_MIN_LENGTH = 1
FIRST_NAME_MAX_LENGTH = 64
LAST_NAME_MAX_LENGTH = 64

# Restrictions for messages
MESSAGE_MIN_LENGTH = 1
MESSAGE_MAX_LENGTH = 4096
# message with media files included
MEDIA_MESSAGE_MAX_LENGTH = 1024

# Restrictions for commands
COMMAND_MIN_LENGTH = 1
COMMAND_MAX_LENGTH = 32
COMMAND_DESCRIPTION_MAX_LENGTH = 256

ChatType = Literal["user", "group", "channel", "bot"]


def _is_valid_char_on_sides(s: str) -> bool:
    """
    Check of the correctness of the first and last characters.
    The first and last character must not be an underscore.
    The first character also can't be a digit.
    """
    if not s:  # empty string
        return False

    first_char = s[0]
    last_char = s[len(s) - 1]
    return not first_char.isdigit() and first_char != "_" and last_char != "_"


def _get_description_length_limit(t: ChatType) -> int:
    """
    Gets the maximum allowed length for the description based on the channel
    type.
    If an invalid type is specified, a ValueError will be thrown.
    """
    if t == "user":
        return DESCRIPTION_USER_MAX_LENGTH
    elif t == "group":
        return DESCRIPTION_GROUP_MAX_LENGTH
    elif t == "channel":
        return DESCRIPTION_CHANNEL_MAX_LENGTH
    elif t == "bot":
        return DESCRIPTION_BOT_MAX_LENGTH
    else:
        raise ValueError(f"Unexpected chat type: {t}")


def is_valid_id(_id: str) -> bool:
    return (
        _is_valid_char_on_sides(_id)
        and "__" not in _id
        and (ID_MIN_LENGTH <= len(_id) <= ID_MAX_LENGTH)
        and shared.is_valid_id(_id)
    )


def is_reserved_id(_id: str) -> bool:
    """
    Checks whether the ID is reserved.
    Case-insensitive.

    Refer to the :py:const:`RESERVED_IDS` for the complete list of words.

    :return: ``True`` if **ID** is reserved, ``False`` otherwise
    """
    return _id.lower() in RESERVED_IDS


def is_reserved_start_id(_id: str) -> bool:
    """
    Checks if the initial characters of the ID are a reserved word.
    Case-insensitive.

    Refer to the :py:const:`RESERVED_START_IDS` for the complete list of words.

    :return: ``True`` if **ID** starts with a reserved word, ``False`` otherwise
    """
    _id = _id.lower()

    for word in RESERVED_START_IDS:
        if _id.startswith(word):
            return True

    return False


def is_bot_id(_id: str) -> bool:
    return _id.endswith("bot")


def is_valid_description(text: str, *, chat_type: ChatType = "user") -> bool:
    return len(text) <= _get_description_length_limit(chat_type) and text.isprintable()


def is_valid_chat_name(name: str) -> bool:
    return (
        CHAT_NAME_MIN_LENGTH <= len(name) <= CHAT_NAME_MAX_LENGTH and name.isprintable()
    )


def is_valid_first_name(name: str) -> bool:
    return (
        FIRST_NAME_MIN_LENGTH <= len(name) <= FIRST_NAME_MAX_LENGTH
        and name.isprintable()
    )


def is_valid_last_name(name: str) -> bool:
    return len(name) <= LAST_NAME_MAX_LENGTH and name.isprintable()


def is_valid_full_name(first_name: str, last_name: str = "") -> bool:
    return is_valid_first_name(first_name) and is_valid_last_name(last_name)


def is_valid_message(text: str, *, include_media: bool = False) -> bool:
    return (
        MESSAGE_MIN_LENGTH
        <= len(text)
        <= (MESSAGE_MAX_LENGTH if not include_media else MEDIA_MESSAGE_MAX_LENGTH)
        and text.isprintable()
    )


def is_valid_command(cmd: str) -> bool:
    return (COMMAND_MIN_LENGTH <= len(cmd) <= COMMAND_MAX_LENGTH) and (
        shared.is_valid_id(cmd)
    )


def validate_id(_id: str) -> str:
    """
    Validates the text identifier of the user/channel/bot.
    This a public identifier by which you can find the desired chat
    (available by ``t.me/%id%``).

    Only ID characters are allowed: A-Za-z, 0-9 and underscores.
    ID only start with a letter, not contain double underscores, have a length
    of 5 to 32 characters, and must not start with or be one of the reserved words.

    :param _id: User, channel or bot identifier
    :return: Input value, converted to lower-case
    :raises ValidationError: if the above conditions are not met
    """
    if not is_valid_id(_id):
        raise ValidationError(
            "ID must be length from 5 to 32 chars, starts with a letter and consists of: A-Za-z, 0-9 and underscores",
            input_value=_id,
        )

    if is_reserved_id(_id):
        raise ValidationError(
            "ID must not be a reserved word",
            input_value=_id,
        )

    if is_reserved_start_id(_id):
        raise ValidationError(
            "ID must not start with a reserved word",
            input_value=_id,
        )

    return _id.lower()


def validate_bot_id(_id: str) -> str:
    """
    Validates the text identifier and checks the suffix "bot". Only ID
    characters are allowed.

    :param _id: Bot identifier
    :return: Input value, converted to lower-case
    :raises ValidationError: if the passed ID does not have a ``bot`` suffix
        or ID validation was fails
    """
    _id = validate_id(_id)

    if not is_bot_id(_id):
        raise ValidationError("Bot ID must have the suffix 'bot'", input_value=_id)

    return _id


def validate_description(text: str, *, chat_type: ChatType = "user") -> str:
    """
    Validates the description (about) field for the selected chat type.
    All characters are allowed.

    Each type has a different maximum length:
        * user - 70
        * group - 255
        * channel - 255
        * bot - 120

    :param text: biography (about) text
    :param chat_type: type of chat in which the text is located
    :return: Input text
    :raises ValueError: if the specified chat type is invalid
    :raises ValidationError: if the passed text exceeds the maximum allowed
        length or if the text is escaped
    """
    if not is_valid_description(text, chat_type=chat_type):
        length_limit = _get_description_length_limit(chat_type)
        raise ValidationError(
            f"Description text must contain no more than {length_limit} characters"
            f" for '{chat_type}' chat type and must not be escaped",
            input_value=text,
        )

    return text


def validate_chat_name(name: str) -> str:
    """
    Validates the name of channel or group for the minimum (1) and maximum (128)
    length of characters. All characters are allowed.

    :param name: Channel or group name
    :return: Input name
    :raises ValidationError: if the name has insufficient characters,
        exceeds the maximum allowed length, or if the name is escaped
    """
    if not is_valid_chat_name(name):
        raise ValidationError(
            f"Chat name must have a character limit of {CHAT_NAME_MIN_LENGTH} to "
            f"{CHAT_NAME_MAX_LENGTH} and mustn't be escaped",
            input_value=name,
        )

    return name


def validate_full_name(*, first_name: str, last_name: str = "") -> Tuple[str, str]:
    """
    Validates the first and last name separately for the length limit.
    All characters are allowed.
    Passed names must not exceed 64 characters.
    The first name can't be empty, the last name, on the contrary, can.

    :param first_name: User's first name
    :param last_name: User's last name
    :return: Tuple of input parameters (first_name, last_name)
    :raises ValidationError: if the passed name has insufficient characters,
        exceeds the maximum allowed length, or the name is escaped
    """
    if not is_valid_first_name(first_name):
        raise ValidationError(
            f"First name must have a character limit of {FIRST_NAME_MIN_LENGTH} "
            f"to {FIRST_NAME_MAX_LENGTH} and must not be escaped",
            input_value=first_name,
        )
    if not is_valid_last_name(last_name):
        raise ValidationError(
            f"Last name must not exceed {LAST_NAME_MAX_LENGTH} characters and"
            f" must not be escaped",
            input_value=last_name,
        )

    return first_name, last_name


def validate_message(text: str, *, include_media: bool = False) -> str:
    """
    Validates the text message from user/channel/bot. All characters are
    allowed. The length must always be at least one character. The maximum
    length is calculated from the include_media condition if it is true, then
    the maximum length will be 1024, otherwise 4096.

    :param text: Message text
    :param include_media: Does the message additionally includes media files
    :return: Input text
    :raises ValidationError: if the text is empty or the length limit is
        exceeded, or if the text is escaped
    """
    if not is_valid_message(text, include_media=include_media):
        raise ValidationError(
            f"Message must contain from {MESSAGE_MIN_LENGTH} to "
            f"{MESSAGE_MAX_LENGTH if not include_media else MEDIA_MESSAGE_MAX_LENGTH} "
            f"characters with include_media={include_media} and must not be escaped",
            input_value=text,
        )

    return text


def validate_command(cmd: str) -> str:
    """
    Validates the bot text command for the minimum (1) and maximum (32) length.
    Only ID characters are allowed: A-Za-z, 0-9 and underscores.

    Please note: Each command is stored in lower-case, but for a more concise
    implementation, uppercase characters are also available for you. They will
    be converted to the lower-case

    :param cmd: Bot text command
    :return: Input command, converted to lower-case
    :raises ValidationError: if the command is empty, or the length limit is
        exceeded, or contains characters that are not allowed by Telegram.
    """
    if not is_valid_command(cmd):
        raise ValidationError(
            f"Command must contain from {COMMAND_MIN_LENGTH} to {COMMAND_MAX_LENGTH} "
            f"characters and consist of: A-Za-z, 0-9 and underscores",
            input_value=cmd,
        )

    return cmd.lower()



import pytest as pytest
from assertpy import assert_that

from social_validator import telegram
from social_validator.exceptions import ValidationError
from tests.telegram import input

# *** Validation functions ***


@pytest.mark.parametrize("_id", input.VALID_IDS)
def test_valid_id(_id: str) -> None:
    v = telegram.validate_id(_id)
    assert_that(v).is_equal_to(_id.lower())


@pytest.mark.parametrize("_id", input.INVALID_IDS)
def test_invalid_id(_id: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_id(_id)


@pytest.mark.parametrize("_id", input.VALID_BOT_IDS)
def test_valid_bot_id(_id: str) -> None:
    v = telegram.validate_bot_id(_id)
    assert_that(v).is_equal_to(_id.lower())


@pytest.mark.parametrize("_id", input.INVALID_BOT_IDS)
def test_invalid_bot_id(_id: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_bot_id(_id)


@pytest.mark.parametrize("_id", input.RESERVED_IDS)
def test_reserved_id(_id: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_id(_id)

    with pytest.raises(ValidationError):
        telegram.validate_bot_id(_id)


@pytest.mark.parametrize("_id", input.RESERVED_START_IDS)
def test_reserved_start_id(_id: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_id(_id)

    with pytest.raises(ValidationError):
        telegram.validate_bot_id(_id)


@pytest.mark.parametrize("text, chat_type", input.VALID_DESCRIPTIONS)
def test_valid_description(text: str, chat_type: telegram.ChatType) -> None:
    v = telegram.validate_description(text, chat_type=chat_type)
    assert_that(v).is_equal_to(text)


@pytest.mark.parametrize("text, chat_type", input.INVALID_DESCRIPTIONS)
def test_invalid_description(text: str, chat_type: telegram.ChatType) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_description(text, chat_type=chat_type)


@pytest.mark.parametrize("text, chat_type", input.INVALID_DESCRIPTIONS_CHAT_TYPES)
def test_invalid_chat_type(text: str, chat_type: telegram.ChatType) -> None:
    with pytest.raises(ValueError):
        telegram.validate_description(text, chat_type=chat_type)


@pytest.mark.parametrize("name", input.VALID_CHAT_NAMES)
def test_valid_chat_name(name: str) -> None:
    v = telegram.validate_chat_name(name)
    assert_that(v).is_equal_to(name)


@pytest.mark.parametrize("name", input.INVALID_CHAT_NAMES)
def test_invalid_chat_name(name: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_chat_name(name)


@pytest.mark.parametrize("first_name, last_name", input.VALID_FULL_NAMES)
def test_valid_full_name(first_name: str, last_name: str) -> None:
    out_first_name, out_last_name = telegram.validate_full_name(
        first_name=first_name, last_name=last_name
    )
    assert_that(out_first_name).is_equal_to(first_name)
    assert_that(out_last_name).is_equal_to(last_name)


@pytest.mark.parametrize("first_name, last_name", input.INVALID_FULL_NAMES)
def test_invalid_full_name(first_name: str, last_name: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_full_name(first_name=first_name, last_name=last_name)


@pytest.mark.parametrize("text, include_media", input.VALID_MESSAGES)
def test_valid_message(text: str, include_media: bool) -> None:
    v = telegram.validate_message(text, include_media=include_media)
    assert_that(v).is_equal_to(text)


@pytest.mark.parametrize("text, include_media", input.INVALID_MESSAGES)
def test_invalid_message(text: str, include_media: bool) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_message(text, include_media=include_media)


@pytest.mark.parametrize("cmd", input.VALID_COMMANDS)
def test_valid_command(cmd: str) -> None:
    v = telegram.validate_command(cmd)
    assert_that(v).is_equal_to(cmd.lower())


@pytest.mark.parametrize("cmd", input.INVALID_COMMANDS)
def test_invalid_command(cmd: str) -> None:
    with pytest.raises(ValidationError):
        telegram.validate_command(cmd)


# *** Check functions ***


@pytest.mark.parametrize("first_name, last_name", input.VALID_FULL_NAMES)
def test_is_valid_full_name(first_name: str, last_name: str) -> None:
    assert_that(telegram.is_valid_full_name(first_name, last_name)).is_true()


@pytest.mark.parametrize("first_name, last_name", input.INVALID_FULL_NAMES)
def test_is_invalid_full_name(first_name: str, last_name: str) -> None:
    assert_that(telegram.is_valid_full_name(first_name, last_name)).is_false()



from social_validator import youtube
from tests.shared.input import ESCAPED_STRING, RANDOM_UNICODE_STRING

VALID_STRICT_USERNAMES = (
    "t" * youtube.USERNAME_MIN_LENGTH,
    "t" * youtube.USERNAME_MAX_LENGTH,
    "1test",
    "1test1",
    "1test-",
    "1TEST-",
    "1test-----",
    "1test-----____",
    "1test-----____----",
    "1test-----___asd_--123--",
    "1test-----___asd_--.123--",
    "1test-----___asd_....123--",
    "-------___asd_....123--",
    "__-----___asd_....123--",
    "__-----___asd_....123--",
    "__--____",
    "__-",
    "___",
    "---",
    "1.....",
    "2.....----____",
    "----____......",
)
VALID_SOFT_USERNAMES = (
    ('-' * 100) + 'tes',
    ('-' * 100) + 'test',
    ('-' * 100) + 'TEST',
    ('-' * 100) + 'test.',
    ('-' * 100) + 'test___----',
    ('-' * 100) + 'test' + ('-' * 100),
    ('-' * 100) + 'test' + ('_' * 100),
    ('-' * 100) + '1..' + ('_' * 100),
    ('-' * 100) + '2-._._-' + ('_' * 100),
)
INVALID_SOFT_USERNAMES = (
    ('_' * 100) + ('a' * (youtube.USERNAME_MIN_LENGTH - 1)),
    ('-' * 100) + ('a' * (youtube.USERNAME_MAX_LENGTH + 1)),
    ('_' * 100) + ('a' * (youtube.USERNAME_MAX_LENGTH + 1)),
    ('-' * 100) + ('a' * (youtube.USERNAME_MIN_LENGTH - 1)),
    ('_' * 100) + ('1' * (youtube.USERNAME_MIN_LENGTH - 1)),
    ('-' * 100) + ('1' * (youtube.USERNAME_MAX_LENGTH + 1)),
    RANDOM_UNICODE_STRING,
    ESCAPED_STRING,
)
INVALID_STRICT_USERNAMES = (
    '',
    't' * (youtube.USERNAME_MIN_LENGTH - 1),
    't' * (youtube.USERNAME_MAX_LENGTH + 1),
    '.' * youtube.USERNAME_MIN_LENGTH,
    '......1',
    '.____',
    '.----',
    '.____----',
    '.____----123',
    '.____----123test',
    RANDOM_UNICODE_STRING,
    ESCAPED_STRING,
)

VALID_VIDEO_IDS = (
    '1asdfgher14',
    '1asdfgher--',
    '1asdfgh__--',
    '-asdfgh__--',
    '_asdfgh__--',
    '___----__--',
    '___----__-1',
    '___----__-a',
    '___----__a1',
    '___--a-__a1',
)
INVALID_VIDEO_IDS = (
    'a' * (youtube.VIDEO_ID_LENGTH - 1),
    'a' * (youtube.VIDEO_ID_LENGTH + 1),
    'a.12ijd2h3j',
    'a=12ijd2h3j',
    'a+12ijd2h3j',
    'a!12ijd2h3j',
    'a@12ijd2h3j',
    'a#12ijd2h3j',
    'a$12ijd2h3j',
    'a%12ijd2h3j',
    'a&12ijd2h3j',
    'a?12ijd2h3j',
    'a*12ijd2h3j',
    'a<>()d2h3j',
    RANDOM_UNICODE_STRING[:youtube.VIDEO_ID_LENGTH],
    (ESCAPED_STRING * 2)[:youtube.VIDEO_ID_LENGTH],
)

VALID_VIDEO_NAMES = (
    'a' * youtube.VIDEO_NAME_MIN_LENGTH,
    'a' * youtube.VIDEO_NAME_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)
INVALID_VIDEO_NAMES = (
    'a' * (youtube.VIDEO_NAME_MIN_LENGTH - 1),
    'a' * (youtube.VIDEO_NAME_MAX_LENGTH + 1),
    '<',
    '>',
    ESCAPED_STRING,
)

VALID_VIDEO_DESCRIPTIONS = (
    '',
    'a' * youtube.VIDEO_DESCRIPTION_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)
VALID_VIDEO_DESCRIPTIONS_SHORT = (
    '',
    'a' * youtube.VIDEO_DESCRIPTION_SHORT_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)
INVALID_VIDEO_DESCRIPTIONS = (
    'a' * (youtube.VIDEO_DESCRIPTION_MAX_LENGTH + 1),
    '<',
    '>',
    ESCAPED_STRING,
)
INVALID_VIDEO_DESCRIPTIONS_SHORT = (
    'a' * (youtube.VIDEO_DESCRIPTION_SHORT_MAX_LENGTH + 1),
    '<',
    '>',
    ESCAPED_STRING,
)

VALID_VIDEO_COMMENTS = (
    'a' * youtube.VIDEO_COMMENT_MIN_LENGTH,
    'a' * youtube.VIDEO_COMMENT_MAX_LENGTH,
    RANDOM_UNICODE_STRING,
)
INVALID_VIDEO_COMMENTS = (
    'a' * (youtube.VIDEO_COMMENT_MIN_LENGTH - 1),
    'a' * (youtube.VIDEO_COMMENT_MAX_LENGTH + 1),
    ESCAPED_STRING,
)

# combine a lower and upper case
RESERVED_USERNAMES = list(youtube.RESERVED_USERNAMES) + [w.upper() for w in youtube.RESERVED_USERNAMES]



from typing import Any


class ValidationError(Exception):
    detail: str
    input_value: Any

    def __init__(self, detail: str, *, input_value: Any, **kwargs: Any) -> None:
        self.detail = detail
        self.input_value = input_value
        self.arguments = kwargs
